@@ -1,7 +1,7 @@
 /*
 curs_spec.c
 
-Copyright 2009, 2010, 2011 Free Software Foundation, Inc.
+Copyright 2009, 2010, 2011, 2014 Free Software Foundation, Inc.
 
 This file is part of GNU Guile-Ncurses.
 
@@ -28,10 +28,12 @@ License along with Guile-Ncurses.  If not, see
 
 #if HAVE_CURSES_H
 #include <curses.h>
-#endif
-
-#if HAVE_NCURSES_CURSES_H
+#elif HAVE_NCURSES_CURSES_H
 #include <ncurses/curses.h>
+#elif HAVE_NCURSESW_CURSES_H
+#include <ncursesw/curses.h>
+#else
+#error "No curses.h file included"
 #endif
 
 #ifdef GUILE_CHARS_ARE_UCS4
